@@ -44,22 +44,22 @@ WINDOW_AGGREGATES = ["row_number", "rank", "dense_rank", "percent_rank",
                      "cume_dist", "ntile", "lag", "lead", "first_value",
                      "last_value", "nth_value"]
 
-# timeUnits from vega-lite version 2.4.3
+# timeUnits from vega-lite version 3.0.0
 TIMEUNITS = ["utcyear", "utcquarter", "utcmonth", "utcday", "utcdate",
              "utchours", "utcminutes", "utcseconds", "utcmilliseconds",
              "utcyearquarter", "utcyearquartermonth", "utcyearmonth",
              "utcyearmonthdate", "utcyearmonthdatehours",
              "utcyearmonthdatehoursminutes",
              "utcyearmonthdatehoursminutesseconds",
-             "utcquartermonth", "utcmonthdate", "utchoursminutes",
-             "utchoursminutesseconds", "utcminutesseconds",
+             "utcquartermonth", "utcmonthdate", "utcmonthdatehours",
+             "utchoursminutes", "utchoursminutesseconds", "utcminutesseconds",
              "utcsecondsmilliseconds",
              "year", "quarter", "month", "day", "date", "hours", "minutes",
              "seconds", "milliseconds", "yearquarter", "yearquartermonth",
              "yearmonth", "yearmonthdate", "yearmonthdatehours",
              "yearmonthdatehoursminutes",
              "yearmonthdatehoursminutesseconds", "quartermonth", "monthdate",
-             "hoursminutes", "hoursminutesseconds", "minutesseconds",
+             "monthdatehours", "hoursminutes", "hoursminutesseconds", "minutesseconds",
              "secondsmilliseconds"]
 
 
@@ -249,18 +249,19 @@ def parse_shorthand(shorthand, data=None, parse_aggregates=True,
 
     units = dict(field='(?P<field>.*)',
                  type='(?P<type>{})'.format('|'.join(valid_typecodes)),
-                 count='(?P<aggregate>count)',
+                 agg_count='(?P<aggregate>count)',
+                 op_count='(?P<op>count)',
                  aggregate='(?P<aggregate>{})'.format('|'.join(AGGREGATES)),
                  window_op='(?P<op>{})'.format('|'.join(AGGREGATES + WINDOW_AGGREGATES)),
                  timeUnit='(?P<timeUnit>{})'.format('|'.join(TIMEUNITS)))
 
     patterns = []
 
-    if parse_aggregates or parse_window_ops:
-        patterns.extend([r'{count}\(\)'])
     if parse_aggregates:
+        patterns.extend([r'{agg_count}\(\)'])
         patterns.extend([r'{aggregate}\({field}\)'])
     if parse_window_ops:
+        patterns.extend([r'{op_count}\(\)'])
         patterns.extend([r'{window_op}\({field}\)'])
     if parse_timeunits:
         patterns.extend([r'{timeUnit}\({field}\)'])
@@ -270,7 +271,7 @@ def parse_shorthand(shorthand, data=None, parse_aggregates=True,
     if parse_types:
         patterns = list(itertools.chain(*((p + ':{type}', p) for p in patterns)))
 
-    regexps = (re.compile('\A' + p.format(**units) + '\Z', re.DOTALL)
+    regexps = (re.compile(r'\A' + p.format(**units) + r'\Z', re.DOTALL)
                for p in patterns)
 
     # find matches depending on valid fields passed
@@ -319,7 +320,7 @@ def use_signature(Obj):
             except AttributeError:
                 # __doc__ is not modifiable for classes in Python < 3.3
                 pass
-        
+
         return f
     return decorate
 
